<?php

/////////////////////////////////////////////////
////// BULK NO-PLAN USERS ASSOCIATION ///////////
/////////////////////////////////////////////////

function pcpp_bulk_no_plan_assoc() {
	if(!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {die('Cheating?');};
	include_once(PCPP_DIR .'/functions.php');
	global $pc_users, $pc_meta, $pcpp_subscr;
	
	$plans = $pcpp_subscr->get_plans(false);
	$plan_id = trim(addslashes($_POST['plan_id'])); 
	if (!isset($plans[$plan_id])) {die( __('Wrong or missing plan ID', 'pcpp_ml') );}

	$use_trial = (int)$_POST['use_trial'];
	
	// query all users without a plan
	$args = array(
		'to_get' 	=> array('id', 'name', 'surname', 'username', 'email', 'tel', 'wp_user_id'),
		'limit' 	=> -1,
		'search' => array(
			array('key'=>'pcpp_plan', 'operator'=>'NOT IN', 'val'=>array_keys($plans))
		)
	);
	$users_data = $pc_users->get_users($args);
	
	
	foreach($users_data as $ud) {
	
		// create order
		$order_id = $pcpp_subscr->create_order($ud['id'], $plan_id);	
		if(!$order_id) {
			continue;
		}
		
		// set new main order and manage access metas
		$pcpp_subscr->manag_user_orders($ud['id'], $order_id, 'add');
		
		// eventually setup trial
		if($use_trial) {
			$pcpp_subscr->setup_trial_meta($ud['id']);
		}
	}

	echo 'success';
	die();	
}
add_action('wp_ajax_pcpp_bulk_no_plan_assoc', 'pcpp_bulk_no_plan_assoc');




///////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////
////// ADD USER - ASSOCIATE USER WITH NO PLAN ///
/////////////////////////////////////////////////

function pcpp_no_plan_associate() {
	if (!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {die('Cheating?');};
	include_once(PCPP_DIR .'/functions.php');
	global $pc_meta, $pcpp_subscr;
	
	$user_id = trim(addslashes($_POST['user_id'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {die( __('Error processing the action', 'pcpp_ml') );}
	
	$plans = $pcpp_subscr->get_plans(false);
	$plan_id = trim(addslashes($_POST['plan_id'])); 
	if (!isset($plans[$plan_id])) {die( __('Wrong or missing plan ID', 'pcpp_ml') );}

	$use_trial = (int)$_POST['use_trial'];
	
	// create order
	$order_id = $pcpp_subscr->create_order($user_id, $plan_id);	
	if(!$order_id) {
		die( __('Error creating new order', 'pcpp_ml') );
	}
	
	// set new main order and manage access metas
	$pcpp_subscr->manag_user_orders($user_id, $order_id, 'add');
	
	// eventually setup trial
	if($use_trial) {
		$pcpp_subscr->setup_trial_meta($user_id);
	}

	echo 'success';
	die();	
}
add_action('wp_ajax_pcpp_no_plan_associate', 'pcpp_no_plan_associate');




///////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////
////// EDIT USER - CHANGE USER PLAN /////////////
/////////////////////////////////////////////////

function pcpp_change_user_plan() {
	if (!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {$error = 'Cheating?';};
	include_once(PCPP_DIR .'/functions.php');
	global $pc_meta, $pcpp_subscr;
	
	$user_id = trim(addslashes($_POST['user_id'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {$error = __('Error processing the action', 'pcpp_ml');}
	
	
	// dissociates from plans
	if($_POST['plan_id'] == 'no_plan') {
		if(isset($error)) {
			die( json_encode(array(
				'status' => 'error', 
				'error' => $error
			)));	
		}
		
		$pc_meta->delete_meta($user_id, 'pcpp_plan');	
		$pc_meta->delete_meta($user_id, 'pcpp_subscr_status');	
		$pc_meta->delete_meta($user_id, 'pcpp_subscr_end');	
		
		die( json_encode(array(
			'status' 		=> 'success', 
			))
		);
	}
	
	
	// normal plan change
	$plan_id = trim(addslashes($_POST['plan_id'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {$error = __('Wrong or missing plan ID', 'pcpp_ml');}
	
	// check among valid plans
	$plans = $pcpp_subscr->get_plans(false);
	if(!isset($plans[$plan_id])) {$error = __('Wrong or missing plan ID', 'pcpp_ml');}

	// responses
	if(isset($error)) {
		die( json_encode(array(
			'status' => 'error', 
			'error' => $error
		)));	
	}
	else {
		// set old order to cancelled if not paid  
		$curr_order = $pc_meta->get_meta($user_id, 'pcpp_active_order');
		if($curr_order) {
			$pcpp_subscr->cancel_order($curr_order, __('Manual subscription plan change'));	
		}

		// create new order for new plan
		$new_order_id = $pcpp_subscr->create_order($user_id, $plan_id);
	
		// set new main order and manage access metas
		$pcpp_subscr->manag_user_orders($user_id, $new_order_id, 'add');
		
		// remove expired plan's notification flag
		$pc_meta->delete_meta($user_id, 'pcpp_expier_subscr_notified');
		
		
		die( json_encode(array(
			'status' => 'success'
		)));		
	}	
}
add_action('wp_ajax_pcpp_change_user_plan', 'pcpp_change_user_plan');



/////////////////////////////////////////////////////
////// EDIT USER - CHANGE USER SUBSCRIPTION STATUS //
/////////////////////////////////////////////////////

function pcpp_change_user_plan_status() {
	if (!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {$error = 'Cheating?';};
	include_once(PCPP_DIR .'/functions.php');
	global $pc_meta, $pcpp_subscr;
	
	$user_id = trim(addslashes($_POST['user_id'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {$error =  __('Error processing the action', 'pcpp_ml');}
	
	$user_plan = trim(addslashes($_POST['user_plan'])); 
	if (!filter_var($user_plan, FILTER_VALIDATE_INT)) {$error =  __('User plan is missing', 'pcpp_ml');}
	
	$status = trim(addslashes($_POST['status'])); 
	if (!in_array($status, array_keys(pcpp_subscr_statuses())) ) {$error = __('Wrong or missing status', 'pcpp_ml');}
	
	
	$is_unlimited = 0;
	$ending_date = false;
	$ending_html = false;
	

	// if activate - update expiration date
	if($status == 'active') {
		$plan_data = $pcpp_subscr->get_plan_data($user_plan);
		$user_expir = $pc_meta->get_meta($user_id, 'pcpp_subscr_end');
		$curr_order = $pc_meta->get_meta($user_id, 'pcpp_active_order');
		
		if(empty($plan_data['duration'])) {
			$is_unlimited = 1;
			
			if(empty($user_expir)) {
				$pc_meta->update_meta($fdata['user_id'], 'pcpp_subscr_end', 'unlimited');	
			}
		}
		else {
			// start period from now
			$ending = pcpp_get_ending_date($plan_data['duration'], $plan_data['duration_type']);	
			$pc_meta->update_meta($user_id, 'pcpp_subscr_end', $ending);	
			
			$ending_date = date('Y-m-d', $ending);
			$ending_html = pcpp_time_to_date_html($ending);
		}
		
		
		// if has unpaid order associated - cancel it
		if($curr_order) {
			$pcpp_subscr->cancel_order($curr_order, __('Manual subscription activation'));
		}
	}
	
	// pending status - only if user has order and latest one isn't active
	elseif($status == 'unpaid') {
		$order_id = $pc_meta->get_meta($user_id, 'pcpp_active_order');
		
		if(empty($order_id)) {
			$error = __('Cannot set as unpaid - no associated order', 'pcpp_ml');			
		}
		else {
			$order_status = $pcpp_subscr->get_order_status($order_id);
			
			if($order_status == 'active') {
				$error = __('Cannot set as unpaid - active order is already paid', 'pcpp_ml');	
			}
			elseif($order_status == 'expired') {
				// active order isn't active - set to unpaid
				if(PCPP_ECOMMERCE == 'woocomm') {
					$order = $pcpp_subscr->get_order($order_id);
					$order->update_status('pending', __('Subscription manually set as unpaid', 'pcpp_ml').' -', true);	
				}
			}
		}
	}
	
	
	// expired status - if user has got pending order, cancel it
	elseif($status == 'expired') {
		$order_id = $pc_meta->get_meta($user_id, 'pcpp_active_order');
		
		if(!empty($order_id) && $pcpp_subscr->get_order_status($order_id) == 'unpaid') {
			if(PCPP_ECOMMERCE == 'woocomm') {
				$order = $pcpp_subscr->get_order($order_id);
				$order->update_status('cancelled', __('Subscription manually set as expired', 'pcpp_ml').' -', true);	
			}
		}
	}


	if(isset($error)) {
		die( json_encode(array(
			'status' => 'error', 
			'error' => $error
		)));	
	}
	else {
		$pc_meta->update_meta($user_id, 'pcpp_subscr_status', $status);
		
		// remove expired plan's notification flag
		$pc_meta->delete_meta($user_id, 'pcpp_expier_subscr_notified');
		
		die( json_encode(array(
			'status' 		=> 'success', 
			'is_unl_plan' 	=> $is_unlimited,
			'ending_date'	=> $ending_date,
			'ending_html'	=> $ending_html
		)));		
	}
}
add_action('wp_ajax_pcpp_change_user_plan_status', 'pcpp_change_user_plan_status');



/////////////////////////////////////////////////////
/// EDIT USER - CHANGE USER SUBSCR EXPIRATION DATE //
/////////////////////////////////////////////////////

function pcpp_change_user_plan_exp_date() {
	if (!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {$error = 'Cheating?';};
	
	include_once(PC_DIR .'/classes/simple_form_validator.php');
	include_once(PCPP_DIR .'/functions.php');
	global $pc_meta, $pcpp_subscr;
	
	$validator = new simple_fv;	
	$indexes = array();
	$indexes[] = array('index'=>'user_id', 'label'=>__('User ID', 'pcpp_ml'), 'type'=>'int', 'required'=>true);
	$indexes[] = array('index'=>'date', 'label'=>__('Expiration date', 'pcpp_ml'), 'type'=>'iso_date');
	$indexes[] = array('index'=>'time', 'label'=>__('Expiration time', 'pcpp_ml'), 'required'=>true);
	$indexes[] = array('index'=>'unlimited', 'label'=>__('unlimited plan?', 'pcpp_ml'), 'type'=>'int', 'required'=>true);
	
	$validator->formHandle($indexes);
	$fdata = $validator->form_val;
	$errors = $validator->getErrors();
	
	// or unlimited or valid date must be specified
	if(empty($errors)) {
		if(empty($fdata['unlimited']) && empty($fdata['date'])) {
			$errors = __('Specify a valid date or set it to unlimited', 'pcpp_ml');	
		}
	}
	
	// calculate ending and check it's not before now
	$ending = (empty($fdata['unlimited'])) ? strtotime($fdata['date'] .' '. $fdata['time'] .':00') : 'unlimited';
	if($ending != 'unlimited' && (int)$ending <= (int)current_time('timestamp')) {
		$errors .= __('Chosen date is already expired', 'pcpp_ml');	
	}
	
	// responses
	if(!empty($errors)) {
		die( json_encode(array(
			'status' => 'error', 
			'error' => $errors
		)));	
	}
	else {
		
		// save meta
		$pc_meta->update_meta($fdata['user_id'], 'pcpp_subscr_end', $ending);
	
		// remove expired plan's notification flag
		$pc_meta->delete_meta($user_id, 'pcpp_expier_subscr_notified');
	
	
		die( json_encode(array(
			'status' 		=> 'success', 
			'ending_html'	=> pcpp_time_to_date_html($ending)
		)));		
	}	
}
add_action('wp_ajax_pcpp_change_user_plan_exp_date', 'pcpp_change_user_plan_exp_date');



/////////////////////////////////////////////////
////// EDIT USER - DELETE ORDER /////////////////
/////////////////////////////////////////////////

function pcpp_del_user_order() {
	if (!$GLOBALS['pcpp_enabled'] || !isset($_POST['pcpp_nonce']) || !wp_verify_nonce($_POST['pcpp_nonce'], 'lcwp_ajax')) {die('Cheating?');};
	global $pcpp_subscr;
	
	$user_id = trim(addslashes($_POST['user_id'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {$error =  __('Error processing the action', 'pcpp_ml');}
	
	$order_id = trim(addslashes($_POST['order_id'])); 
	if (!filter_var($order_id, FILTER_VALIDATE_INT)) {die( __('Missing order ID', 'pcpp_ml') );}
	
	
	$pcpp_subscr->manag_user_orders($user_id, $order_id, 'remove', false); 
	wp_delete_post($order_id, true);

	echo 'success';
	die();	
}
add_action('wp_ajax_pcpp_del_user_order', 'pcpp_del_user_order');