<?php

//////////////////////////////////////////////////////
//////////////// COUPON CHECK ////////////////////////
//////////////////////////////////////////////////////

function pcpp_coupon_check() {
	if(!$GLOBALS['pcpp_enabled'] || !isset($_POST['type']) || $_POST['type'] != 'pcpp_coupon_check') {return false;}	

	include_once(PCPP_DIR .'/functions.php');
	global $pcpp_subscr;
	
	// check coupon existence
	if(!isset($_POST['pcpp_coupon']) || empty($_POST['pcpp_coupon'])) {
		die(json_encode( array('status' => 'error', 'mess' => __('Please insert a valid coupon', 'pcpp_ml')) ));
	}
	$coupon = (string)$_POST['pcpp_coupon'];
	
	
	// match coupon against plans
	if(PCPP_ECOMMERCE == 'woocomm') {
		$coupon_inst = new WC_Coupon($coupon);
		$coup_amount = $coupon_inst->get_amount();
		
		if(empty($coup_amount)) {
			die(json_encode( array('status' => 'error', 'mess' => __('Coupon not found', 'pcpp_ml')) ));	
		}
		
		// calculate new prices for each plan
		$new_prices = array();
		foreach( $pcpp_subscr->get_plans() as $plan_id => $name) {
			
			$product = new WC_Product($plan_id);
			$price = $product->get_price();
			$currency_price_template = str_replace(wc_format_localized_price($price), 'X', strip_tags($product->get_price_html()));

			if($coupon_inst->is_valid_for_product($product) && $price >= $coupon_inst->get_minimum_amount() && (!$coupon_inst->get_maximum_amount() || $price <= $coupon_inst->get_maximum_amount()) ) {	
				$discount_amount = $coupon_inst->get_discount_amount($price, null, true);
				
				// 100% discount?
				if(!round($price - $discount_amount, 2)) {
					$new_prices[$plan_id] = __('free', 'pcpp_ml');
				}
				else {
					$new_price = wc_format_localized_price( round($price - $discount_amount, 2) );	
					$new_prices[$plan_id] = str_replace('X', $new_price, $currency_price_template);
				}
			}
		}
		
		
		// no discount available
		if(empty($new_prices)) {
			die(json_encode( array('status' => 'error', 'mess' => __("Sorry, this coupon isn't valid for these subscriptions", 'pcpp_ml')) ));		
		}
		else {
			
			// there's at least one subscription, ok
			die(json_encode( array('status' => 'success', 'prices' => $new_prices) ));		
		}
	}
	
	die();
}
add_action('wp', 'pcpp_coupon_check', 2); // use this hook to allow constants setup




//////////////////////////////////////////////////////
// RENEW SUBSCRIPTION - OPTIONALLY SPECIFY NEW PLAN //
//////////////////////////////////////////////////////

function pcpp_ajax_renew_subscription() {
	if(!$GLOBALS['pcpp_enabled'] || !isset($_POST['type']) || $_POST['type'] != 'pcpp_renew_order') {return false;}	

	include_once(PCPP_DIR .'/functions.php');
	global $pc_users, $pc_meta, $pcpp_subscr;
	
	$user_id = trim(addslashes($_POST['uid'])); 
	if (!filter_var($user_id, FILTER_VALIDATE_INT)) {$error = __('Error processing the action', 'pcpp_ml');}
	
	// get user plan
	$plan_id = $pc_meta->get_meta($user_id, 'pcpp_plan');
	if(empty($plan_id)) {$error = __('User not found', 'pcpp_ml');}
	
	// get user subscription status
	$subscr_status = $pc_meta->get_meta($user_id, 'pcpp_subscr_status');
	if(empty($subscr_status) || $subscr_status != 'expired') {$error = __("Your subscription isn't expired", 'pcpp_ml');}
	
	// if user plan is specified - otherwise use current one
	if(isset($_POST['pid'])) {
		$plans = $pcpp_subscr->get_plans(false, true);
		$plan_id = (int)$_POST['pid']; 
		if (!isset($plans[$plan_id])) {$error = __('Wrong or missing plan ID', 'pcpp_ml');}
	}
	
	// create order
	if(!isset($error)) {
		$coupon = (isset($_POST['coupon']) && !empty($_POST['coupon'])) ? (string)$_POST['coupon'] : false;
		$order_id = $pcpp_subscr->create_order($user_id, $plan_id, array(), $coupon);
			
		if(!$order_id) {
			$error = __('Error creating new order - contact site administrator', 'pcpp_ml');
		}
		
		// set new main order and manage access metas
		$pcpp_subscr->manag_user_orders($user_id, $order_id, 'add');	
		
		// remove expired plan's notification flag
		$pc_meta->delete_meta($user_id, 'pcpp_expier_subscr_notified');
	}
	
	
	// json to return
	if(isset($error)) {
		die( json_encode(array(
			'status' => 'error', 
			'error' => $error
		)));	
	}	
	else {
		// create
	
		echo json_encode(array(
			'status' => 'success',
			'url' => $pcpp_subscr->checkout_url($order_id),
		));	
		die();
	}
}
add_action('wp', 'pcpp_ajax_renew_subscription', 2); // use this hook to allow constants setup
