<?php
// PRIVATECONTENT INTEGRATIONS - FRONT SIDE

// USER PERMISSIONS - check if allowed categories are associated to plans and match with user
// overrides pvtContent checks if a plan is involved in allows or blocks
function pcpp_extra_user_check($to_return, $user_cats, $allowed, $blocked) {
	global $pc_meta, $pcpp_subscr;
	$user_id = $GLOBALS['pc_user_id'];
		
	// if pcpp disabled - do nothing
	if(!$GLOBALS['pcpp_enabled']) {return $to_return;}

	
	// allow involves also PCPP?
	$allowed_pcpp = array();
	foreach($allowed as $cat) {
		if(strpos($cat, 'PCPP_') !== false) {
			$allowed_pcpp[] = str_replace('PCPP_', '', $cat);	
		}
	}
	
	// block involves also PCPP?
	$blocked_pcpp = array();
	foreach($blocked as $cat) {
		if(strpos($cat, 'PCPP_') !== false) {
			$blocked_pcpp[] = str_replace('PCPP_', '', $cat);	
		}
	}
	
	// not involved - skip check
	if(empty($allowed_pcpp) && empty($blocked_pcpp)) {return $to_return;}
	

	// get meta and cache it to avoid multi calls
	if(isset($GLOBALS['pcpp_euc_user_plan'])) {
		$user_plan = $GLOBALS['pcpp_euc_user_plan']; 	
	} else {
		$user_plan = $pc_meta->get_meta($user_id, 'pcpp_plan');
		$GLOBALS['pcpp_euc_user_plan'] = $user_plan;	
	}
	
	/////
	
	// filter allows
	if(!empty($allowed_pcpp)) {
		$to_return = (in_array($user_plan, array_unique($allowed_pcpp))) ? 1 : 2;		
	}
	
	// filter blocks
	if(!empty($blocked_pcpp) && in_array($user_plan, array_unique($blocked_pcpp)) ) {
		$to_return = 2;		
	}
	
	return $to_return;
}
add_filter('pc_extra_user_check', 'pcpp_extra_user_check', 10, 4);




// PERSISTENT LOGGED USERS CHECK - every 15 minutes
function pcpp_logged_check_cron() {
	if(!is_admin() && $GLOBALS['pcpp_enabled'] && !isset($GLOBALS['pcpp_already_checked_login']) && pc_user_logged(false)) {
		global $pc_meta, $pcpp_subscr;
		$user_id = $GLOBALS['pc_user_id'];
		
		// only for users associated with plans
		if(!$pc_meta->get_meta($user_id, 'pcpp_plan')) {return true;}
		
		// control last check date
		$last_check = (int)$pc_meta->get_meta($user_id, 'pcpp_last_check'); 
		$curr_time = (int)current_time('timestamp');

		if(($curr_time - $last_check) > (60 /* 15*/) ) {
			$check = $pcpp_subscr->check_user_subscr_expir($user_id);
			if($check == 'unpaid' || $check == 'no_act_order' || $check == 'expired') {
				pc_logout();  // logout
			}
			
			$pc_meta->update_meta($user_id, 'pcpp_last_check', $curr_time); 
		}
	}
}
add_action('wp', 'pcpp_logged_check_cron', 100); // low priority to let PC doing its works and PCPP constant setup




// LOGIN CHECK - subscription checks
function pcpp_login_check($cust_mess, $user_id) {
	global $pc_users, $pc_meta, $pcpp_subscr;

	// if there's already a message - return it
	if(!empty($cust_mess)) {return $cust_mess;}
	
	// if pcpp disabled - allow
	if(!$GLOBALS['pcpp_enabled']) {return false;}
	
	// global to avoid persistent logged user check
	$GLOBALS['pcpp_already_checked_login'] = true;
	
	// check subscription
	$check = $pcpp_subscr->check_user_subscr_expir($user_id);
	if($check === true || $check == 'active' || $check == 'trial') {return false;}
	
	
	// UNPAID - default or expired trial mess
	if($check == 'unpaid') {
		// setup checkout URL
		$active_order = $pc_meta->get_meta($user_id, 'pcpp_active_order');
		$checkout_url = ($active_order) ? $pcpp_subscr->checkout_url($active_order) : '';
		
		if(isset($GLOBALS['pcpp_expired_trial'])) { // expired trial
			$exp_trial_mess = get_option('pcpp_exp_trial_mex');
			if(empty($exp_trial_mess)) {$exp_trial_mess = __('Hello, your trial period ended', 'pcpp_ml');}
			$exp_trial_mess .= ' - <a href="'.$checkout_url.'">'. __('pay now', 'pcpp_ml') .'</a>';
			
			return $exp_trial_mess;
		} 
		else {
			$unpaid_mess = get_option('pcpp_unpaid_mex');
			if(empty($unpaid_mess)) {$unpaid_mess = __('Hello, your subscription is unpaid', 'pcpp_ml');}
			$unpaid_mess .= ' - <a href="'. $checkout_url .'">'. __('pay now', 'pcpp_ml') .'</a>'; 
			
			return $unpaid_mess;
		}
	}

	elseif($check == 'no_act_order' || $check == 'expired') {
		$renew_pag = get_option('pcpp_order_renew_page');
		
		//// if allow to change plan, link to specific page - otherwise inform and add code for JS handler
		// code for handler
		if(empty($renew_pag)) {
			$link_url = 'javascript:void(0)'; 
			$link_params = 'class="pcpp_renew_plan_btn" rel="'.$user_id.'"';
		}
		
		// link to change plan page
		else {
			$link_url = add_query_arg(array(
					'pcpp_action' => 'order_ok',
					'uid' => $user_id,
					'oid' => md5( $pc_meta->get_meta($user_id, 'pcpp_active_order') ), 
				), 
				get_permalink($renew_pag)
			);	
			$link_params = '';
		}


		// compose and return message
		$exp_plan_mess = get_option('pcpp_expired_mex');
		if(empty($exp_plan_mess)) {
			$exp_plan_mess = __('Hello, your subscription expired', 'pcpp_ml');
		}
		
		// renewal check
		$user_plan = $pc_meta->get_meta($user_id, 'pcpp_plan');
		if(
			empty($user_plan) ||
			(get_post_type($user_plan) == 'product' && !get_post_meta($user_plan, 'pcpp_no_renewal', true)) // woocommerce
		) {
			$exp_plan_mess .= ' - <a href="'.$link_url.'" '.$link_params.'>'. __('renew it', 'pcpp_ml') .'</a>';		
		}
		
		
		return $exp_plan_mess;
	}
	
	return $cust_mess;
}
add_filter('pc_login_custom_check', 'pcpp_login_check', 999, 2);




////////////////////////////////////////////////////////////////////////




// add custom field with custom field type
function pcpp_add_reg_field($fields) {
	
	$fields['pcpp_plan'] = array(
		'label' 	=> __('Subscription plan', 'pcpp_ml'),
		'type' 		=> 'pcpp_plan_block',
		'sys_req' 	=> true,
	); 
	return $fields;
}
add_filter('pc_form_fields_filter', 'pcpp_add_reg_field');



// declare forbidden field indexes
function pcpp_forbidden_field_indexes($fields) {
	return array_merge($fields, array('pcpp_plan', 'pcpp_coupon'));
}
add_filter('pc_forbidden_field_indexes', 'pcpp_forbidden_field_indexes');


// field must not be used in wizards
function pcpp_subscr_field_no_wizard($fields) {
	return array_merge($fields, array('pcpp_plan'));
}
add_filter('pc_no_wizard_indexes', 'pcpp_subscr_field_no_wizard');



// add field type
function pcpp_reg_field_code($code, $type, $field_data, $user_id) {
	if(!$GLOBALS['pcpp_enabled'] || $type != 'pcpp_plan_block') {return $code;}

	include_once(PCPP_DIR .'/functions.php');
	global $pc_meta, $pcpp_subscr;
	
	$strip_no_registr = (isset($GLOBALS['pcpp_renewal_form'])) ? false : true;
	$strip_no_renewal = (isset($GLOBALS['pcpp_renewal_form'])) ? true : false;
	
	$plans = $pcpp_subscr->get_plans($strip_no_registr, $strip_no_renewal);
	
	
	// trial text
	if($pcpp_subscr->is_trial_enabled()) {
		$trial_txt = get_option('pcpp_trial_txt');
		if(empty($trial_txt)) {$trial_txt = '<strong>'. $pcpp_subscr->trial_duration.' '. pcpp_sing_plur_switch($pcpp_subscr->trial_dur_type, $pcpp_subscr->trial_duration) .' '. __('free trial', 'pcpp_ml') .'!</strong>';}
		
		$trial_box = '<div class="pcpp_trial_box">'. $trial_txt .'</div>';	
	}
	else {$trial_box = '';}
	
	
	// separator first - use disclaimer's wrap code
	$code .= '
	<section class="pc_rf_disclaimer_sep pcpp_field_top_sep"></section>
	<section class="pc_rf_disclaimer pcpp_field pcpp_plan_choice_wrap">';
		
	// only one plan: just print plan's specifications
	if(count($plans) == 1) {
		$keys = array_keys($plans);
		$plan_id = $keys[0];
		
		$vals = array_values($plans);
		$plan_name = $vals[0];
		
		$data = $pcpp_subscr->get_plan_data($plan_id);
		
		$code .= '
		<label id="pcpp_plan_dd_label">'. __('Subscription details', 'pcpp_ml') .':</label>
		'. $trial_box .'
		
		<ul class="pcpp_plan_ul pc_plan_dd_single">
			<li class="pcpp_plan_'.$plan_id.' pcpp_chosen" rel="'.$plan_id.'">'. pcpp_plan_form_txt($data, $plan_name) .'</li>
		</ul>
		
		<input type="hidden" name="pcpp_plan" value="'. $plan_id .'" />
		';
	}
	else {
		$bold_layout = (get_option('pcpp_bold_fluid_layout')) ? 'pcpp_bold_fluid_layout' : '';
		
		$code .= '
		<label id="pcpp_plan_dd_label">'. __('Choose your plan', 'pcpp_ml') .': <i></i></label>
		'. $trial_box .'
		
		<ul class="pcpp_plan_ul '.$bold_layout.'">';
		
		if(!empty($user_id)) {
			$sel_to_match = $pc_meta->get_meta($user_id, 'pcpp_plan');
		} 
		if(!isset($sel_to_match) || !isset($plans[$sel_to_match])) {
			$sel_to_match = (isset($_REQUEST['pcpp_plan'])) ? (int)$_REQUEST['pcpp_plan'] : (int)get_option('pcpp_default_plan', 0);
		}
		
		$sel_plan = '';
		$a = 0;
		foreach($plans as $plan_id => $plan_name) {
			$data = $pcpp_subscr->get_plan_data($plan_id);
			$sel = ($plan_id == $sel_to_match || (!$sel_to_match && !$a)) ? 'class="pcpp_chosen"' : '';
			if(!empty($sel)) {$sel_plan = $plan_id;}
			
			$code .= '<li rel="'.$plan_id.'" '.$sel.'>'. pcpp_plan_form_txt($data, $plan_name) .'</li>';
			$a++;
		}
		
		$code .= '</ul>
		<select name="pcpp_plan" class="pcpp_plan_dd" autocomplete="off">';
		
		foreach($plans as $plan_id => $plan_name) {
			$sel = ($plan_id == $sel_plan) ? 'selected="selected"' : '';
			$code .= '<option value="'. $plan_id .'" '.$sel.'>'. $plan_name .'</option>';	
		}
		
		$code .= '</select>';
	}
	
	
	// allow coupon usage?
	if(get_option('pcpp_use_coupon')) {
		
		$coupon_label = __('Got a coupon to use?', 'pcpp_ml');
		
		// fields icon
		$coupon_icon_class 	= (get_option('pcpp_coupon_field_icon')) ? 'pc_field_w_icon' : '';
		$coupon_icon 		= ($coupon_icon_class) ? '<span class="pc_field_icon" title="'. esc_attr($coupon_label) .'"><i class="fa '.get_option('pcpp_coupon_field_icon').'"></i></span>' : '';
		
		// placeholders only if no-label is active
		$coupon_placeh = (get_option('pg_nolabel')) ? 'placeholder="'. esc_attr($coupon_label) .'"' : '';
		
		
		$code .= '
		</section>
		
		<section class="pc_rf_field pc_rf_disclaimer pcpp_coupon_wrap '. $coupon_icon_class .'">
			<div class="pcpp_coupon_box">
				<label>'. $coupon_label .'</label>
				
				<div class="pc_field_container">
					'. $coupon_icon .'
					<input class="pcpp_coupon" name="pcpp_coupon" value="" maxlength="30" '. $coupon_placeh .' autocomplete="off" type="text" />
					
					<i class="fa fa-chevron-right pcpp_try_coupon" title="'. esc_attr(__('Apply coupon', 'pcpp_ml')) .'"></i>
				</div>
			</div>
		';	
	}
	
	
	// add code to change submit button value
	$custom_btn_val = get_option('pcpp_reg_form_btn_text');
	if(!empty($custom_btn_val)) {
		$code .= '
		<script type="text/javascript">
		jQuery(document).ready(function() {
			jQuery(".pc_reg_btn").attr("value", "'. addslashes($custom_btn_val) .'");
		});
		</script>';
	}
	
	// JS code to manage switch - use JS file
	return $code . '</section>';
}
add_filter('pc_custom_field_type', 'pcpp_reg_field_code', 10, 4); 


// always add into registration form
function pcpp_add_field_into_reg_forms($structure, $form_id) {
    if(!get_option('pcpp_enabled', false)) {return $structure;} // constant hasn't been declared at this point
	include_once(PCPP_DIR .'/functions.php');
	
	$structure['include'][] = 'pcpp_plan';
    $structure['require'][] = 'pcpp_plan';
    
    return $structure;
}
add_filter('pc_registration_form', 'pcpp_add_field_into_reg_forms', 9999, 2); 




////////////////////////////////////////////////////////////////////////////////////




// REGISTRATION PROCESSES

// registration validation - check field value
function pcpp_reg_form_check_val($errors, $fdata) {
	if(!get_option('pcpp_enabled', false)) {return $errors;} // constant hasn't been declared at this point
	
	if(isset($fdata['pcpp_plan']) && !empty($fdata['pcpp_plan'])) {
		
		include_once(PCPP_DIR .'/functions.php');
		global $pcpp_subscr;
		
		$allowed_plans = array_keys($pcpp_subscr->get_plans());
		
		// allow "no_plan" for admin insertion and import 
		if(strpos($_SERVER["REQUEST_URI"], '?page=pc_add_user') === false || strpos($_SERVER["REQUEST_URI"], '?page=pc_import_export') === false) {
			$allowed_plans[] = 'no_plan';	
		}

		if( !in_array((string)$fdata['pcpp_plan'], $allowed_plans) ) {
			$errors[] = __('Please select a valid subscription plan', 'pcpp_ml');	
		}
	}
	return $errors;
}
add_filter('pc_form_valid_errors', 'pcpp_reg_form_check_val', 10, 2);




// create order or just setup metas for trial periods
function pcpp_reg_to_subscr($user_id) {
	include_once(PCPP_DIR .'/functions.php');
	global $pc_users, $pc_meta, $pcpp_subscr;
	$metas = array();
	
	if(!get_option('pcpp_enabled', false)) {return false;} // constant hasn't been declared at this point
	
	
	// get user data to perform actions
	$user_data = $pc_users->get_user($user_id, $args = array('to_get' => array('name', 'surname', 'username', 'email', 'tel', 'wp_user_id', 'pcpp_plan')));
	
	// get plan specifications
	$plan_data = $pcpp_subscr->get_plan_data($user_data['pcpp_plan']); // plan is saved through normal registration
	
	// create order
	$coupon = (isset($_POST['pcpp_coupon']) && !empty($_POST['pcpp_coupon'])) ? (string)$_POST['pcpp_coupon'] : false;
	$order_id = $pcpp_subscr->create_order($user_id, $user_data['pcpp_plan'], $user_data, $coupon);	
	if(!$order_id) {
		// error creating the order - flag to customize successful message 	
		$GLOBALS['pcpp_mess_customizer'] = 'registr_order_failed';
		return true;
	}

	// set new main order and manage access metas
	$pcpp_subscr->manag_user_orders($user_id, $order_id, 'add');
	
	// eventually setup trial
	$pcpp_subscr->setup_trial_meta($user_id);
	
	// NOT-FREE AND NOT TRIAL - customize redirect and message
	if( !$pcpp_subscr->is_trial_enabled() && !empty($plan_data['price']) && !isset($GLOBALS['pcpp_free_order_due_to_coupon']) ) {
		$GLOBALS['pcpp_mess_customizer'] = 'registr_order_ok';
		$GLOBALS['pcpp_redir_customizer'] = 'registr_order_ok';
		$GLOBALS['pcpp_redirect_to_use'] = $pcpp_subscr->checkout_url($order_id);
		
		// customize form redirect
		add_filter('pc_custom_redirect_url', 'pcpp_customize_pc_redirect', 999, 2);	
		
		// customize form message
		add_filter('pc_customize_message', 'pcpp_customize_pc_mess', 999, 2);
	}		
	
	return true;
}
add_action('pc_registered_user', 'pcpp_reg_to_subscr', 100); 



////////////////////////////////////////////////////////////////////////////////////



//// function used to customize privateContent messages - 
// called by different hooks and return string depending on pcpp_mess_customizer global
function pcpp_customize_pc_mess($mess, $subj) {
	if(!isset($GLOBALS['pcpp_mess_customizer'])) {return $mess;}
	$pcpp_subj = $GLOBALS['pcpp_mess_customizer'];
	
	// registration
	if($subj == 'pc_default_sr_mex') {
		if($pcpp_subj == 'registr_order_failed') {
			 $mess = __('Registration was successful but an error occurred creating your subscription order. Please contact the site admin', 'pcpp_ml');	
		}
		else if($pcpp_subj == 'registr_order_ok') {
			 $mess = get_option('pcpp_registr_mex');	
			 if(empty($mess)) {$mess = __('Successful registration! Now you will be moved to checkout page', 'pcpp_ml');}
		}
	}

	return $mess;	
}


//// function used to customize privateContent redirects - 
// called by different hooks and return string depending on pcpp_redir_customizer global
function pcpp_customize_pc_redirect($url, $key) {
	if(!isset($GLOBALS['pcpp_redir_customizer'])) {return $url;}
	$pcpp_subj = $GLOBALS['pcpp_redir_customizer'];
	
	// registration
	if($key == 'pc_registered_user_redirect') {
		if($pcpp_subj == 'registr_order_ok') {
			 $url = $GLOBALS['pcpp_redirect_to_use'];
		}
	}
	
	return $url;	
}


////////////////////////////////////////////////////////////////////////////////////


// remove PvtContent plans from visible page and queries
function pcpp_hide_plans($bool, $post) {
	if(is_admin()) {return false;}
	
	// woocommerce
	if($GLOBALS['pcpp_enabled'] && $post->post_type == 'product' && function_exists('wc_get_product')) {
		$product = wc_get_product($post->ID);
		
		// Woocomm v3 compatibility
		if(method_exists($product, 'get_type')) {
			if($product->get_type() == 'pvtcontent_plan') {
				return true;	
			}		
		} 
		else {
			if($product->product_type == 'pvtcontent_plan') {
				return true;	
			}
		}
	}
	
	return false;
}
add_filter('pc_forced_page_restriction', 'pcpp_hide_plans', 1, 2);


function pcpp_query_filter($query) {
	global $wpdb, $pcpp_subscr;

	if(is_admin() || !$GLOBALS['pcpp_enabled'] || isset($GLOBALS['pcpp_is_querying plans'])) {return $query;}
	
	$GLOBALS['pcpp_no_wpml_integration'] = true; // get current language IDs
	$plans = array_keys($pcpp_subscr->get_plans(false));
	
	if(!empty($plans)) {
		$query->set('post__not_in', $plans);	
	}
	
	return $query;
}
add_filter('pre_get_posts', 'pcpp_query_filter', 9999);




// remove landing pages from complete site lock
function pcpp_complete_lock_exceptions($excluded_pages) {
	if(!$GLOBALS['pcpp_enabled']) {return $excluded_pages;}
	
	$exceptions = array();
	
	// TODO - exclude e-commerce operational system ??
	
	// pcpp landing pages
	$exceptions[] = (int)get_option('pcpp_order_ok_page');
	$exceptions[] = (int)get_option('pcpp_order_del_page');
	$exceptions[] = (int)get_option('pcpp_order_renew_page');
	
	return (is_array($excluded_pages)) ? array_merge($excluded_pages, $exceptions) : $exceptions;
}
add_filter('pc_complete_lock_exceptions', 'pcpp_complete_lock_exceptions', 10);



// disable direct opening for pcpp landing pages
function pcpp_landing_page_no_direct_access() {
	if(is_page() && $GLOBALS['pcpp_enabled']) {
		$matches = array();
		$matches[] = (int)get_option('pcpp_order_ok_page');
		$matches[] = (int)get_option('pcpp_order_del_page');
		$matches[] = (int)get_option('pcpp_order_renew_page');
		
		$curr_pag_id = (int)get_the_ID();
		
		foreach($matches as $pag_id) {
			if(!empty($pag_id)) {
				if($curr_pag_id == $pag_id && !isset($_REQUEST['pcpp_action'])) {
					header('location: '.get_home_url());
					die();		
				}
			}
		}
	}
}
add_action('template_redirect', 'pcpp_landing_page_no_direct_access');